// Copyright 2004 Marc Plouhinec
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
ION83EMUPC par Marc Plouhinec (m_plouhinec@yahoo.fr    www.thebestof.tk)
Emulateur ION83EMU
C'est un mulateur en 2 parties, une sur le PC qui modifie les jeux pour les rendre compatible avec la TI-82
et l'autre qui l'adapte pour SNG.
Ce logiciel est la 1re partie de l'mulateur.

Tches du programme:
1-dterminer ce qui est donne et ce qui est instruction en stockant les liens (sauts, appels, LD,...)
  vers la rom et la ram.
2-modifier ces liens pour les adapter sur la TI-82
3-cration du fichier .82p

Date de cration: 19/07/2004
Dernire mise  jour: 08/08/2004
*/

#include "stdafx.h"
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <STDLIB.H>
#include <fcntl.h>

#define PROGSTART 0x9327

bool DebugMode = false;
bool AdresseTrouvee;
FILE* fichier_83p;
FILE* fichier_82p;
short TailleProgrammeASM;
short AdresseRelativePremInstr;
char Programme[28000];
char OmbreDuProgramme[28000];
char AdresseLDBC[28000];
char Nom[50];
fpos_t pos;

//Definition des 519 instructions
static unsigned char RefInstructions[519][6] = {
	//Taille | Instruction | Special (0 = rien; 1 = JP; 2 = JR ou DJNZ; 3=CALL; 4=LD+16bits; 5=RET)
	{1,0x8E,0   ,0   ,0   ,0}, //ADC  A,(HL)  8E   1 NOP 1
	{3,0x8E,0xDD,0   ,0   ,0}, //ADC  A,(IX*) 8EDD 3 ZIX 1
	{3,0x8E,0xFD,0   ,0   ,0}, //ADC  A,(IY*) 8EFD 3 ZIX 1
	{1,0x8F,0   ,0   ,0   ,0}, //ADC  A,A     8F   1 NOP 1
	{1,0x88,0   ,0   ,0   ,0}, //ADC  A,B     88   1 NOP 1
	{1,0x89,0   ,0   ,0   ,0}, //ADC  A,C     89   1 NOP 1
	{1,0x8A,0   ,0   ,0   ,0}, //ADC  A,D     8A   1 NOP 1
	{1,0x8B,0   ,0   ,0   ,0}, //ADC  A,E     8B   1 NOP 1
	{1,0x8C,0   ,0   ,0   ,0}, //ADC  A,H     8C   1 NOP 1
	{1,0x8D,0   ,0   ,0   ,0}, //ADC  A,L     8D   1 NOP 1
	{2,0xCE,0   ,0   ,0   ,0}, //ADC  A,*     CE   2 NOP 1
	{2,0x4A,0xED,0   ,0   ,0}, //ADC  HL,BC   4AED 2 NOP 1
	{2,0x5A,0xED,0   ,0   ,0}, //ADC  HL,DE   5AED 2 NOP 1
	{2,0x6A,0xED,0   ,0   ,0}, //ADC  HL,HL   6AED 2 NOP 1
	{2,0x7A,0xED,0   ,0   ,0}, //ADC  HL,SP   7AED 2 NOP 1

	{1,0x86,0   ,0   ,0   ,0}, //ADD  A,(HL)  86   1 NOP 1
	{3,0x86,0xDD,0   ,0   ,0}, //ADD  A,(IX*) 86DD 3 ZIX 1
	{3,0x86,0xFD,0   ,0   ,0}, //ADD  A,(IY*) 86FD 3 ZIX 1
	{1,0x87,0   ,0   ,0   ,0}, //ADD  A,A     87   1 NOP 1
	{1,0x80,0   ,0   ,0   ,0}, //ADD  A,B     80   1 NOP 1
	{1,0x81,0   ,0   ,0   ,0}, //ADD  A,C     81   1 NOP 1
	{1,0x82,0   ,0   ,0   ,0}, //ADD  A,D     82   1 NOP 1
	{1,0x83,0   ,0   ,0   ,0}, //ADD  A,E     83   1 NOP 1
	{1,0x84,0   ,0   ,0   ,0}, //ADD  A,H     84   1 NOP 1
	{1,0x85,0   ,0   ,0   ,0}, //ADD  A,L     85   1 NOP 1
	{2,0xC6,0   ,0   ,0   ,0}, //ADD  A,*     C6   2 NOP 1
	{1,0x09,0   ,0   ,0   ,0}, //ADD  HL,BC   09   1 NOP 1
	{1,0x19,0   ,0   ,0   ,0}, //ADD  HL,DE   19   1 NOP 1
	{1,0x29,0   ,0   ,0   ,0}, //ADD  HL,HL   29   1 NOP 1
	{1,0x39,0   ,0   ,0   ,0}, //ADD  HL,SP   39   1 NOP 1
	{2,0x09,0xDD,0   ,0   ,0}, //ADD  IX,BC   09DD 2 NOP 1
	{2,0x19,0xDD,0   ,0   ,0}, //ADD  IX,DE   19DD 2 NOP 1
	{2,0x29,0xDD,0   ,0   ,0}, //ADD  IX,IX   29DD 2 NOP 1
	{2,0x39,0xDD,0   ,0   ,0}, //ADD  IX,SP   39DD 2 NOP 1
	{2,0x09,0xFD,0   ,0   ,0}, //ADD  IY,BC   09FD 2 NOP 1
	{2,0x19,0xFD,0   ,0   ,0}, //ADD  IY,DE   19FD 2 NOP 1
	{2,0x29,0xFD,0   ,0   ,0}, //ADD  IY,IY   29FD 2 NOP 1
	{2,0x39,0xFD,0   ,0   ,0}, //ADD  IY,SP   39FD 2 NOP 1

	{1,0xA6,0   ,0   ,0   ,0}, //AND  (HL)    A6   1 NOP 1
	{3,0xA6,0xDD,0   ,0   ,0}, //AND  (IX*)   A6DD 3 ZIX 1
	{3,0xA6,0xFD,0   ,0   ,0}, //AND  (IY*)   A6FD 3 ZIX 1
	{1,0xA7,0   ,0   ,0   ,0}, //AND  A       A7   1 NOP 1
	{1,0xA0,0   ,0   ,0   ,0}, //AND  B       A0   1 NOP 1
	{1,0xA1,0   ,0   ,0   ,0}, //AND  C       A1   1 NOP 1
	{1,0xA2,0   ,0   ,0   ,0}, //AND  D       A2   1 NOP 1
	{1,0xA3,0   ,0   ,0   ,0}, //AND  E       A3   1 NOP 1
	{1,0xA4,0   ,0   ,0   ,0}, //AND  H       A4   1 NOP 1
	{1,0xA5,0   ,0   ,0   ,0}, //AND  L       A5   1 NOP 1
	{2,0xE6,0   ,0   ,0   ,0}, //AND  *       E6   2 NOP 1

	{2,0x46,0xCB,0   ,0   ,0}, //BIT  *,(HL)  46CB 2 ZBIT 1
	{4,0xCB,0xDD,0x46,0x00,0}, //BIT  *,(IX*) CBDD 4 ZBIT 1 0 4600
	{4,0xCB,0xFD,0x46,0x00,0}, //BIT  *,(IY*) CBFD 4 ZBIT 1 0 4600
	{2,0x47,0xCB,0   ,0   ,0}, //BIT  *,A     47CB 2 ZBIT 1
	{2,0x40,0xCB,0   ,0   ,0}, //BIT  *,B     40CB 2 ZBIT 1
	{2,0x41,0xCB,0   ,0   ,0}, //BIT  *,C     41CB 2 ZBIT 1
	{2,0x42,0xCB,0   ,0   ,0}, //BIT  *,D     42CB 2 ZBIT 1
	{2,0x43,0xCB,0   ,0   ,0}, //BIT  *,E     43CB 2 ZBIT 1
	{2,0x44,0xCB,0   ,0   ,0}, //BIT  *,H     44CB 2 ZBIT 1
	{2,0x45,0xCB,0   ,0   ,0}, //BIT  *,L     45CB 2 ZBIT 1

	{3,0xDC,0   ,0   ,0   ,3}, //CALL C,*     DC   3 NOP 1
	{3,0xFC,0   ,0   ,0   ,3}, //CALL M,*     FC   3 NOP 1
	{3,0xD4,0   ,0   ,0   ,3}, //CALL NC,*    D4   3 NOP 1
	{3,0xC4,0   ,0   ,0   ,3}, //CALL NZ,*    C4   3 NOP 1
	{3,0xF4,0   ,0   ,0   ,3}, //CALL P,*     F4   3 NOP 1
	{3,0xEC,0   ,0   ,0   ,3}, //CALL PE,*    EC   3 NOP 1
	{3,0xE4,0   ,0   ,0   ,3}, //CALL PO,*    E4   3 NOP 1
	{3,0xCC,0   ,0   ,0   ,3}, //CALL Z,*     CC   3 NOP 1
	{3,0xCD,0   ,0   ,0   ,3}, //CALL *       CD   3 NOP 1

	{1,0x3F,0   ,0   ,0   ,0}, //CCF  ""      3F   1 NOP 1

	{1,0xBE,0   ,0   ,0   ,0}, //CP   (HL)    BE   1 NOP 1
	{3,0xBE,0xDD,0   ,0   ,0}, //CP   (IX*)   BEDD 3 ZIX 1
	{3,0xBE,0xFD,0   ,0   ,0}, //CP   (IY*)   BEFD 3 ZIX 1
	{1,0xBF,0   ,0   ,0   ,0}, //CP   A       BF   1 NOP 1
	{1,0xB8,0   ,0   ,0   ,0}, //CP   B       B8   1 NOP 1
	{1,0xB9,0   ,0   ,0   ,0}, //CP   C       B9   1 NOP 1
	{1,0xBA,0   ,0   ,0   ,0}, //CP   D       BA   1 NOP 1
	{1,0xBB,0   ,0   ,0   ,0}, //CP   E       BB   1 NOP 1
	{1,0xBC,0   ,0   ,0   ,0}, //CP   H       BC   1 NOP 1
	{1,0xBD,0   ,0   ,0   ,0}, //CP   L       BD   1 NOP 1
	{2,0xFE,0   ,0   ,0   ,0}, //CP   *       FE   2 NOP 1
	{2,0xA9,0xED,0   ,0   ,0}, //CPD  ""      A9ED 2 NOP 1
	{2,0xB9,0xED,0   ,0   ,0}, //CPDR ""      B9ED 2 NOP 1
	{2,0xB1,0xED,0   ,0   ,0}, //CPIR ""      B1ED 2 NOP 1
	{2,0xA1,0xED,0   ,0   ,0}, //CPI  ""      A1ED 2 NOP 1
	{1,0x2F,0   ,0   ,0   ,0}, //CPL  ""      2F   1 NOP 1

	{1,0x27,0   ,0   ,0   ,0}, //DAA  ""      27   1 NOP 1

	{1,0x35,0   ,0   ,0   ,0}, //DEC  (HL)    35   1 NOP 1
	{3,0x35,0xDD,0   ,0   ,0}, //DEC  (IX*)   35DD 3 ZIX 1
	{3,0x35,0xFD,0   ,0   ,0}, //DEC  (IY*)   35FD 3 ZIX 1
	{1,0x3D,0   ,0   ,0   ,0}, //DEC  A       3D   1 NOP 1
	{1,0x05,0   ,0   ,0   ,0}, //DEC  B       05   1 NOP 1
	{1,0x0B,0   ,0   ,0   ,0}, //DEC  BC      0B   1 NOP 1
	{1,0x0D,0   ,0   ,0   ,0}, //DEC  C       0D   1 NOP 1
	{1,0x15,0   ,0   ,0   ,0}, //DEC  D       15   1 NOP 1
	{1,0x1B,0   ,0   ,0   ,0}, //DEC  DE      1B   1 NOP 1
	{1,0x1D,0   ,0   ,0   ,0}, //DEC  E       1D   1 NOP 1
	{1,0x25,0   ,0   ,0   ,0}, //DEC  H       25   1 NOP 1
	{1,0x2B,0   ,0   ,0   ,0}, //DEC  HL      2B   1 NOP 1
	{2,0x2B,0xDD,0   ,0   ,0}, //DEC  IX      2BDD 2 NOP 1
	{2,0x2B,0xFD,0   ,0   ,0}, //DEC  IY      2BFD 2 NOP 1
	{1,0x2D,0   ,0   ,0   ,0}, //DEC  L       2D   1 NOP 1
	{1,0x3B,0   ,0   ,0   ,0}, //DEC  SP      3B   1 NOP 1
	{1,0xF3,0   ,0   ,0   ,0}, //DI   ""      F3   1 NOP 1
	{2,0x10,0   ,0   ,0   ,2}, //DJNZ *       10   2 R1  1

	{1,0xFB,0   ,0   ,0   ,0}, //EI   ""      FB   1 NOP 1
	{1,0xE3,0   ,0   ,0   ,0}, //EX   (SP),HL E3   1 NOP 1
	{2,0xE3,0xDD,0   ,0   ,0}, //EX   (SP),IX E3DD 2 NOP 1
	{2,0xE3,0xFD,0   ,0   ,0}, //EX   (SP),IY E3FD 2 NOP 1
	{1,0x08,0   ,0   ,0   ,0}, //EX   AF,AF'  08   1 NOP 1
	{1,0xEB,0   ,0   ,0   ,0}, //EX   DE,HL   EB   1 NOP 1
	{1,0xD9,0   ,0   ,0   ,0}, //EXX  ""      D9   1 NOP 1
	{1,0x76,0   ,0   ,0   ,0}, //HALT ""      76   1 NOP 1

	{2,0x46,0xED,0   ,0   ,0}, //IM   0       46ED 2 NOP 1
	{2,0x56,0xED,0   ,0   ,0}, //IM   1       56ED 2 NOP 1
	{2,0x5E,0xED,0   ,0   ,0}, //IM   2       5EED 2 NOP 1

	{2,0x78,0xED,0   ,0   ,0}, //IN   A,(C)   78ED 2 NOP 1
	{2,0x40,0xED,0   ,0   ,0}, //IN   B,(C)   40ED 2 NOP 1
	{2,0x48,0xED,0   ,0   ,0}, //IN   C,(C)   48ED 2 NOP 1
	{2,0x50,0xED,0   ,0   ,0}, //IN   D,(C)   50ED 2 NOP 1
	{2,0x58,0xED,0   ,0   ,0}, //IN   E,(C)   58ED 2 NOP 1
	{2,0x60,0xED,0   ,0   ,0}, //IN   H,(C)   60ED 2 NOP 1
	{2,0x68,0xED,0   ,0   ,0}, //IN   L,(C)   68ED 2 NOP 1

	{2,0xDB,0   ,0   ,0   ,0}, //IN   A,(*)   DB   2 NOP 1

	{3,0x38,0xED,0   ,0   ,0}, //IN0   A,(*)  38ED 3 NOP 2
	{3,0x00,0xED,0   ,0   ,0}, //IN0   B,(*)  00ED 3 NOP 2
	{3,0x08,0xED,0   ,0   ,0}, //IN0   C,(*)  08ED 3 NOP 2
	{3,0x10,0xED,0   ,0   ,0}, //IN0   D,(*)  10ED 3 NOP 2
	{3,0x18,0xED,0   ,0   ,0}, //IN0   E,(*)  18ED 3 NOP 2
	{3,0x20,0xED,0   ,0   ,0}, //IN0   H,(*)  20ED 3 NOP 2
	{3,0x28,0xED,0   ,0   ,0}, //IN0   L,(*)  28ED 3 NOP 2

	{1,0x34,0   ,0   ,0   ,0}, //INC  (HL)    34   1 NOP 1
	{3,0x34,0xDD,0   ,0   ,0}, //INC  (IX*)   34DD 3 ZIX 1
	{3,0x34,0xFD,0   ,0   ,0}, //INC  (IY*)   34FD 3 ZIX 1
	{1,0x3C,0   ,0   ,0   ,0}, //INC  A       3C   1 NOP 1
	{1,0x04,0   ,0   ,0   ,0}, //INC  B       04   1 NOP 1
	{1,0x03,0   ,0   ,0   ,0}, //INC  BC      03   1 NOP 1
	{1,0x0C,0   ,0   ,0   ,0}, //INC  C       0C   1 NOP 1
	{1,0x14,0   ,0   ,0   ,0}, //INC  D       14   1 NOP 1
	{1,0x13,0   ,0   ,0   ,0}, //INC  DE      13   1 NOP 1
	{1,0x1C,0   ,0   ,0   ,0}, //INC  E       1C   1 NOP 1
	{1,0x24,0   ,0   ,0   ,0}, //INC  H       24   1 NOP 1
	{1,0x23,0   ,0   ,0   ,0}, //INC  HL      23   1 NOP 1
	{2,0x23,0xDD,0   ,0   ,0}, //INC  IX      23DD 2 NOP 1
	{2,0x23,0xFD,0   ,0   ,0}, //INC  IY      23FD 2 NOP 1
	{1,0x2C,0   ,0   ,0   ,0}, //INC  L       2C   1 NOP 1
	{1,0x33,0   ,0   ,0   ,0}, //INC  SP      33   1 NOP 1

	{2,0xAA,0xED,0   ,0   ,0}, //IND  ""      AAED 2 NOP 1
	{2,0xBA,0xED,0   ,0   ,0}, //INDR ""      BAED 2 NOP 1
	{2,0xA2,0xED,0   ,0   ,0}, //INI  ""      A2ED 2 NOP 1
	{2,0xB2,0xED,0   ,0   ,0}, //INIR ""      B2ED 2 NOP 1

	{1,0xE9,0   ,0   ,0   ,0}, //JP   (HL)    E9   1 NOP 1
	{2,0xE9,0xDD,0   ,0   ,0}, //JP   (IX)    E9DD 2 NOP 1
	{2,0xE9,0xFD,0   ,0   ,0}, //JP   (IY)    E9FD 2 NOP 1
	{3,0xDA,0   ,0   ,0   ,1}, //JP   C,*     DA   3 NOP 1
	{3,0xFA,0   ,0   ,0   ,1}, //JP   M,*     FA   3 NOP 1
	{3,0xD2,0   ,0   ,0   ,1}, //JP   NC,*    D2   3 NOP 1
	{3,0xC2,0   ,0   ,0   ,1}, //JP   NZ,*    C2   3 NOP 1
	{3,0xF2,0   ,0   ,0   ,1}, //JP   P,*     F2   3 NOP 1
	{3,0xEA,0   ,0   ,0   ,1}, //JP   PE,*    EA   3 NOP 1
	{3,0xE2,0   ,0   ,0   ,1}, //JP   PO,*    E2   3 NOP 1
	{3,0xCA,0   ,0   ,0   ,1}, //JP   Z,*     CA   3 NOP 1
	{3,0xC3,0   ,0   ,0   ,1}, //JP   *       C3   3 NOP 1

	{2,0x38,0   ,0   ,0   ,2}, //JR   C,*     38   2 R1  1
	{2,0x30,0   ,0   ,0   ,2}, //JR   NC,*    30   2 R1  1
	{2,0x20,0   ,0   ,0   ,2}, //JR   NZ,*    20   2 R1  1
	{2,0x28,0   ,0   ,0   ,2}, //JR   Z,*     28   2 R1  1
	{2,0x18,0   ,0   ,0   ,2}, //JR   *       18   2 R1  1

	{1,0x02,0   ,0   ,0   ,0}, //LD   (BC),A  02   1 NOP 1
	{1,0x12,0   ,0   ,0   ,0}, //LD   (DE),A  12   1 NOP 1
	{1,0x77,0   ,0   ,0   ,0}, //LD   (HL),A  77   1 NOP 1
	{1,0x70,0   ,0   ,0   ,0}, //LD   (HL),B  70   1 NOP 1
	{1,0x71,0   ,0   ,0   ,0}, //LD   (HL),C  71   1 NOP 1
	{1,0x72,0   ,0   ,0   ,0}, //LD   (HL),D  72   1 NOP 1
	{1,0x73,0   ,0   ,0   ,0}, //LD   (HL),E  73   1 NOP 1
	{1,0x74,0   ,0   ,0   ,0}, //LD   (HL),H  74   1 NOP 1
	{1,0x75,0   ,0   ,0   ,0}, //LD   (HL),L  75   1 NOP 1
	{2,0x36,0   ,0   ,0   ,0}, //LD   (HL),*  36   2 NOP 1
	{3,0x77,0xDD,0   ,0   ,0}, //LD   (IX*),A 77DD 3 ZIX 1
	{3,0x70,0xDD,0   ,0   ,0}, //LD   (IX*),B 70DD 3 ZIX 1
	{3,0x71,0xDD,0   ,0   ,0}, //LD   (IX*),C 71DD 3 ZIX 1
	{3,0x72,0xDD,0   ,0   ,0}, //LD   (IX*),D 72DD 3 ZIX 1
	{3,0x73,0xDD,0   ,0   ,0}, //LD   (IX*),E 73DD 3 ZIX 1
	{3,0x74,0xDD,0   ,0   ,0}, //LD   (IX*),H 74DD 3 ZIX 1
	{3,0x75,0xDD,0   ,0   ,0}, //LD   (IX*),L 75DD 3 ZIX 1
	{4,0x36,0xDD,0   ,0   ,4}, //LD   (IX*),* 36DD 4 ZIX 1
	{3,0x77,0xFD,0   ,0   ,0}, //LD   (IY*),A 77FD 3 ZIX 1
	{3,0x70,0xFD,0   ,0   ,0}, //LD   (IY*),B 70FD 3 ZIX 1
	{3,0x71,0xFD,0   ,0   ,0}, //LD   (IY*),C 71FD 3 ZIX 1
	{3,0x72,0xFD,0   ,0   ,0}, //LD   (IY*),D 72FD 3 ZIX 1
	{3,0x73,0xFD,0   ,0   ,0}, //LD   (IY*),E 73FD 3 ZIX 1
	{3,0x74,0xFD,0   ,0   ,0}, //LD   (IY*),H 74FD 3 ZIX 1
	{3,0x75,0xFD,0   ,0   ,0}, //LD   (IY*),L 75FD 3 ZIX 1
	{4,0x36,0xFD,0   ,0   ,4}, //LD   (IY*),* 36FD 4 ZIX 1
	{3,0x32,0   ,0   ,0   ,4}, //LD   (*),A   32   3 NOP 1
	{4,0x43,0xED,0   ,0   ,4}, //LD   (*),BC  43ED 4 NOP 1
	{4,0x53,0xED,0   ,0   ,4}, //LD   (*),DE  53ED 4 NOP 1
	{3,0x22,0   ,0   ,0   ,4}, //LD   (*),HL  22   3 NOP 1
	{4,0x22,0xDD,0   ,0   ,4}, //LD   (*),IX  22DD 4 NOP 1
	{4,0x22,0xFD,0   ,0   ,4}, //LD   (*),IY  22FD 4 NOP 1
	{4,0x73,0xED,0   ,0   ,4}, //LD   (*),SP  73ED 4 NOP 1
	{1,0x0A,0   ,0   ,0   ,0}, //LD   A,(BC)  0A   1 NOP 1
	{1,0x1A,0   ,0   ,0   ,0}, //LD   A,(DE)  1A   1 NOP 1
	{1,0x7E,0   ,0   ,0   ,0}, //LD   A,(HL)  7E   1 NOP 1
	{3,0x7E,0xDD,0   ,0   ,0}, //LD   A,(IX*) 7EDD 3 ZIX 1
	{3,0x7E,0xFD,0   ,0   ,0}, //LD   A,(IY*) 7EFD 3 ZIX 1
	{1,0x7F,0   ,0   ,0   ,0}, //LD   A,A     7F   1 NOP 1
	{1,0x78,0   ,0   ,0   ,0}, //LD   A,B     78   1 NOP 1
	{1,0x79,0   ,0   ,0   ,0}, //LD   A,C     79   1 NOP 1
	{1,0x7A,0   ,0   ,0   ,0}, //LD   A,D     7A   1 NOP 1
	{1,0x7B,0   ,0   ,0   ,0}, //LD   A,E     7B   1 NOP 1
	{1,0x7C,0   ,0   ,0   ,0}, //LD   A,H     7C   1 NOP 1
	{2,0x57,0xED,0   ,0   ,0}, //LD   A,I     57ED 2 NOP 1
	{1,0x7D,0   ,0   ,0   ,0}, //LD   A,L     7D   1 NOP 1
	{2,0x5F,0xED,0   ,0   ,0}, //LD   A,R     5FED 2 NOP 1
	{3,0x3A,0   ,0   ,0   ,4}, //LD   A,(*)   3A   3 NOP 1
	{2,0x3E,0   ,0   ,0   ,0}, //LD   A,*     3E   2 NOP 1
	{1,0x46,0   ,0   ,0   ,0}, //LD   B,(HL)  46   1 NOP 1
	{3,0x46,0xDD,0   ,0   ,0}, //LD   B,(IX*) 46DD 3 ZIX 1
	{3,0x46,0xFD,0   ,0   ,0}, //LD   B,(IY*) 46FD 3 ZIX 1
	{1,0x47,0   ,0   ,0   ,0}, //LD   B,A     47   1 NOP 1
	{1,0x40,0   ,0   ,0   ,0}, //LD   B,B     40   1 NOP 1
	{1,0x41,0   ,0   ,0   ,0}, //LD   B,C     41   1 NOP 1
	{1,0x42,0   ,0   ,0   ,0}, //LD   B,D     42   1 NOP 1
	{1,0x43,0   ,0   ,0   ,0}, //LD   B,E     43   1 NOP 1
	{1,0x44,0   ,0   ,0   ,0}, //LD   B,H     44   1 NOP 1
	{1,0x45,0   ,0   ,0   ,0}, //LD   B,L     45   1 NOP 1
	{2,0x06,0   ,0   ,0   ,0}, //LD   B,*     06   2 NOP 1
	{4,0x4B,0xED,0   ,0   ,4}, //LD   BC,(*)  4BED 4 NOP 1
	{3,0x01,0   ,0   ,0   ,4}, //LD   BC,*    01   3 NOP 1
	{1,0x4E,0   ,0   ,0   ,0}, //LD   C,(HL)  4E   1 NOP 1
	{3,0x4E,0xDD,0   ,0   ,0}, //LD   C,(IX*) 4EDD 3 ZIX 1
	{3,0x4E,0xFD,0   ,0   ,0}, //LD   C,(IY*) 4EFD 3 ZIX 1
	{1,0x4F,0   ,0   ,0   ,0}, //LD   C,A     4F   1 NOP 1
	{1,0x48,0   ,0   ,0   ,0}, //LD   C,B     48   1 NOP 1
	{1,0x49,0   ,0   ,0   ,0}, //LD   C,C     49   1 NOP 1
	{1,0x4A,0   ,0   ,0   ,0}, //LD   C,D     4A   1 NOP 1
	{1,0x4B,0   ,0   ,0   ,0}, //LD   C,E     4B   1 NOP 1
	{1,0x4C,0   ,0   ,0   ,0}, //LD   C,H     4C   1 NOP 1
	{1,0x4D,0   ,0   ,0   ,0}, //LD   C,L     4D   1 NOP 1
	{2,0x0E,0   ,0   ,0   ,0}, //LD   C,*     0E   2 NOP 1
	{1,0x56,0   ,0   ,0   ,0}, //LD   D,(HL)  56   1 NOP 1
	{3,0x56,0xDD,0   ,0   ,0}, //LD   D,(IX*) 56DD 3 ZIX 1
	{3,0x56,0xFD,0   ,0   ,0}, //LD   D,(IY*) 56FD 3 ZIX 1
	{1,0x57,0   ,0   ,0   ,0}, //LD   D,A     57   1 NOP 1
	{1,0x50,0   ,0   ,0   ,0}, //LD   D,B     50   1 NOP 1
	{1,0x51,0   ,0   ,0   ,0}, //LD   D,C     51   1 NOP 1
	{1,0x52,0   ,0   ,0   ,0}, //LD   D,D     52   1 NOP 1
	{1,0x53,0   ,0   ,0   ,0}, //LD   D,E     53   1 NOP 1
	{1,0x54,0   ,0   ,0   ,0}, //LD   D,H     54   1 NOP 1
	{1,0x55,0   ,0   ,0   ,0}, //LD   D,L     55   1 NOP 1
	{2,0x16,0   ,0   ,0   ,0}, //LD   D,*     16   2 NOP 1
	{4,0x5B,0xED,0   ,0   ,4}, //LD   DE,(*)  5BED 4 NOP 1
	{3,0x11,0   ,0   ,0   ,4}, //LD   DE,*    11   3 NOP 1
	{1,0x5E,0   ,0   ,0   ,0}, //LD   E,(HL)  5E   1 NOP 1
	{3,0x5E,0xDD,0   ,0   ,0}, //LD   E,(IX*) 5EDD 3 ZIX 1
	{3,0x5E,0xFD,0   ,0   ,0}, //LD   E,(IY*) 5EFD 3 ZIX 1
	{1,0x5F,0   ,0   ,0   ,0}, //LD   E,A     5F   1 NOP 1
	{1,0x58,0   ,0   ,0   ,0}, //LD   E,B     58   1 NOP 1
	{1,0x59,0   ,0   ,0   ,0}, //LD   E,C     59   1 NOP 1
	{1,0x5A,0   ,0   ,0   ,0}, //LD   E,D     5A   1 NOP 1
	{1,0x5B,0   ,0   ,0   ,0}, //LD   E,E     5B   1 NOP 1
	{1,0x5C,0   ,0   ,0   ,0}, //LD   E,H     5C   1 NOP 1
	{1,0x5D,0   ,0   ,0   ,0}, //LD   E,L     5D   1 NOP 1
	{2,0x1E,0   ,0   ,0   ,0}, //LD   E,*     1E   2 NOP 1
	{1,0x66,0   ,0   ,0   ,0}, //LD   H,(HL)  66   1 NOP 1
	{3,0x66,0xDD,0   ,0   ,0}, //LD   H,(IX*) 66DD 3 ZIX 1
	{3,0x66,0xFD,0   ,0   ,0}, //LD   H,(IY*) 66FD 3 ZIX 1
	{1,0x67,0   ,0   ,0   ,0}, //LD   H,A     67   1 NOP 1
	{1,0x60,0   ,0   ,0   ,0}, //LD   H,B     60   1 NOP 1
	{1,0x61,0   ,0   ,0   ,0}, //LD   H,C     61   1 NOP 1
	{1,0x62,0   ,0   ,0   ,0}, //LD   H,D     62   1 NOP 1
	{1,0x63,0   ,0   ,0   ,0}, //LD   H,E     63   1 NOP 1
	{1,0x64,0   ,0   ,0   ,0}, //LD   H,H     64   1 NOP 1
	{1,0x65,0   ,0   ,0   ,0}, //LD   H,L     65   1 NOP 1
	{2,0x26,0   ,0   ,0   ,0}, //LD   H,*     26   2 NOP 1
	{3,0x2A,0   ,0   ,0   ,4}, //LD   HL,(*)  2A   3 NOP 1
	{3,0x21,0   ,0   ,0   ,4}, //LD   HL,*    21   3 NOP 1
	{2,0x47,0xED,0   ,0   ,0}, //LD   I,A     47ED 2 NOP 1
	{4,0x2A,0xDD,0   ,0   ,4}, //LD   IX,(*)  2ADD 4 NOP 1
	{4,0x21,0xDD,0   ,0   ,4}, //LD   IX,*    21DD 4 NOP 1
	{4,0x2A,0xFD,0   ,0   ,4}, //LD   IY,(*)  2AFD 4 NOP 1
	{4,0x21,0xFD,0   ,0   ,4}, //LD   IY,*    21FD 4 NOP 1
	{1,0x6E,0   ,0   ,0   ,0}, //LD   L,(HL)  6E   1 NOP 1
	{3,0x6E,0xDD,0   ,0   ,0}, //LD   L,(IX*) 6EDD 3 ZIX 1
	{3,0x6E,0xFD,0   ,0   ,0}, //LD   L,(IY*) 6EFD 3 ZIX 1
	{1,0x6F,0   ,0   ,0   ,0}, //LD   L,A     6F   1 NOP 1
	{1,0x68,0   ,0   ,0   ,0}, //LD   L,B     68   1 NOP 1
	{1,0x69,0   ,0   ,0   ,0}, //LD   L,C     69   1 NOP 1
	{1,0x6A,0   ,0   ,0   ,0}, //LD   L,D     6A   1 NOP 1
	{1,0x6B,0   ,0   ,0   ,0}, //LD   L,E     6B   1 NOP 1
	{1,0x6C,0   ,0   ,0   ,0}, //LD   L,H     6C   1 NOP 1
	{1,0x6D,0   ,0   ,0   ,0}, //LD   L,L     6D   1 NOP 1
	{2,0x2E,0   ,0   ,0   ,0}, //LD   L,*     2E   2 NOP 1
	{2,0x4F,0xED,0   ,0   ,0}, //LD   R,A     4FED 2 NOP 1
	{4,0x7B,0xED,0   ,0   ,4}, //LD   SP,(*)  7BED 4 NOP 1
	{1,0xF9,0   ,0   ,0   ,0}, //LD   SP,HL   F9   1 NOP 1
	{2,0xF9,0xDD,0   ,0   ,0}, //LD   SP,IX   F9DD 2 NOP 1
	{2,0xF9,0xFD,0   ,0   ,0}, //LD   SP,IY   F9FD 2 NOP 1
	{3,0x31,0   ,0   ,0   ,4}, //LD   SP,*    31   3 NOP 1
	{2,0xA8,0xED,0   ,0   ,0}, //LDD  ""      A8ED 2 NOP 1
	{2,0xB8,0xED,0   ,0   ,0}, //LDDR ""      B8ED 2 NOP 1
	{2,0xA0,0xED,0   ,0   ,0}, //LDI  ""      A0ED 2 NOP 1
	{2,0xB0,0xED,0   ,0   ,0}, //LDIR ""      B0ED 2 NOP 1
	{2,0x44,0xED,0   ,0   ,0}, //NEG  ""      44ED 2 NOP 1
	{1,0x00,0   ,0   ,0   ,0}, //NOP  ""      00   1 NOP 1

	{2,0x4C,0xED,0   ,0   ,0}, //MLT BC       4CED 2 NOP 2
	{2,0x5C,0xED,0   ,0   ,0}, //MLT DE       5CED 2 NOP 2
	{2,0x6C,0xED,0   ,0   ,0}, //MLT HL       6CED 2 NOP 2
	{2,0x7C,0xED,0   ,0   ,0}, //MLT SP       7CED 2 NOP 2

	{1,0xB6,0   ,0   ,0   ,0}, //OR   (HL)    B6   1 NOP 1
	{3,0xB6,0xDD,0   ,0   ,0}, //OR   (IX*)   B6DD 3 ZIX 1
	{3,0xB6,0xFD,0   ,0   ,0}, //OR   (IY*)   B6FD 3 ZIX 1
	{1,0xB7,0   ,0   ,0   ,0}, //OR   A       B7   1 NOP 1
	{1,0xB0,0   ,0   ,0   ,0}, //OR   B       B0   1 NOP 1
	{1,0xB1,0   ,0   ,0   ,0}, //OR   C       B1   1 NOP 1
	{1,0xB2,0   ,0   ,0   ,0}, //OR   D       B2   1 NOP 1
	{1,0xB3,0   ,0   ,0   ,0}, //OR   E       B3   1 NOP 1
	{1,0xB4,0   ,0   ,0   ,0}, //OR   H       B4   1 NOP 1
	{1,0xB5,0   ,0   ,0   ,0}, //OR   L       B5   1 NOP 1
	{2,0xF6,0   ,0   ,0   ,0}, //OR   *       F6   2 NOP 1

	{2,0x8B,0xED,0   ,0   ,0}, //OTDM ""      8BED 2 NOP 2
	{2,0x9B,0xED,0   ,0   ,0}, //OTDMR ""     9BED 2 NOP 2
	{2,0xBB,0xED,0   ,0   ,0}, //OTDR ""      BBED 2 NOP 1
	{2,0x83,0xED,0   ,0   ,0}, //OTIM ""      83ED 2 NOP 2
	{2,0x93,0xED,0   ,0   ,0}, //OTIMR ""     93ED 2 NOP 2
	{2,0xB3,0xED,0   ,0   ,0}, //OTIR ""      B3ED 2 NOP 1

	{2,0x79,0xED,0   ,0   ,0}, //OUT  (C),A   79ED 2 NOP 1
	{2,0x41,0xED,0   ,0   ,0}, //OUT  (C),B   41ED 2 NOP 1
	{2,0x49,0xED,0   ,0   ,0}, //OUT  (C),C   49ED 2 NOP 1
	{2,0x51,0xED,0   ,0   ,0}, //OUT  (C),D   51ED 2 NOP 1
	{2,0x59,0xED,0   ,0   ,0}, //OUT  (C),E   59ED 2 NOP 1
	{2,0x61,0xED,0   ,0   ,0}, //OUT  (C),H   61ED 2 NOP 1
	{2,0x69,0xED,0   ,0   ,0}, //OUT  (C),L   69ED 2 NOP 1
	{2,0xD3,0   ,0   ,0   ,0}, //OUT  (*),A   D3   2 NOP 1

	{3,0x39,0xED,0   ,0   ,0}, //OUT0 (*),A   39ED 3 NOP 2
	{3,0x01,0xED,0   ,0   ,0}, //OUT0 (*),B   01ED 3 NOP 2
	{3,0x09,0xED,0   ,0   ,0}, //OUT0 (*),C   09ED 3 NOP 2
	{3,0x11,0xED,0   ,0   ,0}, //OUT0 (*),D   11ED 3 NOP 2
	{3,0x19,0xED,0   ,0   ,0}, //OUT0 (*),E   19ED 3 NOP 2
	{3,0x21,0xED,0   ,0   ,0}, //OUT0 (*),H   21ED 3 NOP 2
	{3,0x29,0xED,0   ,0   ,0}, //OUT0 (*),L   29ED 3 NOP 2

	{2,0xAB,0xED,0   ,0   ,0}, //OUTD ""      ABED 2 NOP 1
	{2,0xA3,0xED,0   ,0   ,0}, //OUTI ""      A3ED 2 NOP 1

	{1,0xF1,0   ,0   ,0   ,0}, //POP  AF      F1   1 NOP 1
	{1,0xC1,0   ,0   ,0   ,0}, //POP  BC      C1   1 NOP 1
	{1,0xD1,0   ,0   ,0   ,0}, //POP  DE      D1   1 NOP 1
	{1,0xE1,0   ,0   ,0   ,0}, //POP  HL      E1   1 NOP 1
	{2,0xE1,0xDD,0   ,0   ,0}, //POP  IX      E1DD 2 NOP 1
	{2,0xE1,0xFD,0   ,0   ,0}, //POP  IY      E1FD 2 NOP 1

	{1,0xF5,0   ,0   ,0   ,0}, //PUSH AF      F5   1 NOP 1
	{1,0xC5,0   ,0   ,0   ,0}, //PUSH BC      C5   1 NOP 1
	{1,0xD5,0   ,0   ,0   ,0}, //PUSH DE      D5   1 NOP 1
	{1,0xE5,0   ,0   ,0   ,0}, //PUSH HL      E5   1 NOP 1
	{2,0xE5,0xDD,0   ,0   ,0}, //PUSH IX      E5DD 2 NOP 1
	{2,0xE5,0xFD,0   ,0   ,0}, //PUSH IY      E5FD 2 NOP 1

	{2,0x86,0xCB,0   ,0   ,0}, //RES  *,(HL)  86CB 2 ZBIT 1
	{4,0xCB,0xDD,0x86,0x00,0}, //RES  *,(IX*) CBDD 4 ZBIT 1 0 8600
	{4,0xCB,0xFD,0x86,0x00,0}, //RES  *,(IY*) CBFD 4 ZBIT 1 0 8600
	{2,0x87,0xCB,0   ,0   ,0}, //RES  *,A     87CB 2 ZBIT 1
	{2,0x80,0xCB,0   ,0   ,0}, //RES  *,B     80CB 2 ZBIT 1
	{2,0x81,0xCB,0   ,0   ,0}, //RES  *,C     81CB 2 ZBIT 1
	{2,0x82,0xCB,0   ,0   ,0}, //RES  *,D     82CB 2 ZBIT 1
	{2,0x83,0xCB,0   ,0   ,0}, //RES  *,E     83CB 2 ZBIT 1
	{2,0x84,0xCB,0   ,0   ,0}, //RES  *,H     84CB 2 ZBIT 1
	{2,0x85,0xCB,0   ,0   ,0}, //RES  *,L     85CB 2 ZBIT 1

	{1,0xC9,0   ,0   ,0   ,5}, //RET  ""      C9   1 NOP 1
	{1,0xD8,0   ,0   ,0   ,5}, //RET  C       D8   1 NOP 1
	{1,0xF8,0   ,0   ,0   ,5}, //RET  M       F8   1 NOP 1
	{1,0xD0,0   ,0   ,0   ,5}, //RET  NC      D0   1 NOP 1
	{1,0xC0,0   ,0   ,0   ,5}, //RET  NZ      C0   1 NOP 1
	{1,0xF0,0   ,0   ,0   ,5}, //RET  P       F0   1 NOP 1
	{1,0xE8,0   ,0   ,0   ,5}, //RET  PE      E8   1 NOP 1
	{1,0xE0,0   ,0   ,0   ,5}, //RET  PO      E0   1 NOP 1
	{1,0xC8,0   ,0   ,0   ,5}, //RET  Z       C8   1 NOP 1
	{2,0x4D,0xED,0   ,0   ,5}, //RETI ""      4DED 2 NOP 1
	{2,0x45,0xED,0   ,0   ,5}, //RETN ""      45ED 2 NOP 1

	{2,0x16,0xCB,0   ,0   ,0}, //RL   (HL)    16CB 2 NOP 1
	{4,0xCB,0xDD,0x16,0x00,0}, //RL   (IX*)   CBDD 4 ZIX 1 0 1600
	{4,0xCB,0xFD,0x16,0x00,0}, //RL   (IY*)   CBFD 4 ZIX 1 0 1600
	{2,0x17,0xCB,0   ,0   ,0}, //RL   A       17CB 2 NOP 1
	{2,0x10,0xCB,0   ,0   ,0}, //RL   B       10CB 2 NOP 1
	{2,0x11,0xCB,0   ,0   ,0}, //RL   C       11CB 2 NOP 1
	{2,0x12,0xCB,0   ,0   ,0}, //RL   D       12CB 2 NOP 1
	{2,0x13,0xCB,0   ,0   ,0}, //RL   E       13CB 2 NOP 1
	{2,0x14,0xCB,0   ,0   ,0}, //RL   H       14CB 2 NOP 1
	{2,0x15,0xCB,0   ,0   ,0}, //RL   L       15CB 2 NOP 1
	{1,0x17,0   ,0   ,0   ,0}, //RLA  ""      17   1 NOP 1

	{2,0x06,0xCB,0   ,0   ,0}, //RLC  (HL)    06CB 2 NOP 1
	{4,0xCB,0xDD,0x06,0x00,0}, //RLC  (IX*)   CBDD 4 ZIX 1 0 0600
	{4,0xCB,0xFD,0x06,0x00,0}, //RLC  (IY*)   CBFD 4 ZIX 1 0 0600
	{2,0x07,0xCB,0   ,0   ,0}, //RLC  A       07CB 2 NOP 1
	{2,0x00,0xCB,0   ,0   ,0}, //RLC  B       00CB 2 NOP 1
	{2,0x01,0xCB,0   ,0   ,0}, //RLC  C       01CB 2 NOP 1
	{2,0x02,0xCB,0   ,0   ,0}, //RLC  D       02CB 2 NOP 1
	{2,0x03,0xCB,0   ,0   ,0}, //RLC  E       03CB 2 NOP 1
	{2,0x04,0xCB,0   ,0   ,0}, //RLC  H       04CB 2 NOP 1
	{2,0x05,0xCB,0   ,0   ,0}, //RLC  L       05CB 2 NOP 1
	{1,0x07,0   ,0   ,0   ,0}, //RLCA ""      07   1 NOP 1
	{2,0x6F,0xED,0   ,0   ,0}, //RLD  ""      6FED 2 NOP 1

	{2,0x1E,0xCB,0   ,0   ,0}, //RR   (HL)    1ECB 2 NOP 1
	{4,0xCB,0xDD,0x1E,0x00,0}, //RR   (IX*)   CBDD 4 ZIX 1 0 1E00
	{4,0xCB,0xFD,0x1E,0x00,0}, //RR   (IY*)   CBFD 4 ZIX 1 0 1E00
	{2,0x1F,0xCB,0   ,0   ,0}, //RR   A       1FCB 2 NOP 1
	{2,0x18,0xCB,0   ,0   ,0}, //RR   B       18CB 2 NOP 1
	{2,0x19,0xCB,0   ,0   ,0}, //RR   C       19CB 2 NOP 1
	{2,0x1A,0xCB,0   ,0   ,0}, //RR   D       1ACB 2 NOP 1
	{2,0x1B,0xCB,0   ,0   ,0}, //RR   E       1BCB 2 NOP 1
	{2,0x1C,0xCB,0   ,0   ,0}, //RR   H       1CCB 2 NOP 1
	{2,0x1D,0xCB,0   ,0   ,0}, //RR   L       1DCB 2 NOP 1
	{1,0x1F,0   ,0   ,0   ,0}, //RRA  ""      1F   1 NOP 1
	{2,0x0E,0xCB,0   ,0   ,0}, //RRC  (HL)    0ECB 2 NOP 1
	{4,0xCB,0xDD,0x0E,0x00,0}, //RRC  (IX*)   CBDD 4 ZIX 1 0 0E00
	{4,0xCB,0xFD,0x0E,0x00,0}, //RRC  (IY*)   CBFD 4 ZIX 1 0 0E00
	{2,0x0F,0xCB,0   ,0   ,0}, //RRC  A       0FCB 2 NOP 1
	{2,0x08,0xCB,0   ,0   ,0}, //RRC  B       08CB 2 NOP 1
	{2,0x09,0xCB,0   ,0   ,0}, //RRC  C       09CB 2 NOP 1
	{2,0x0A,0xCB,0   ,0   ,0}, //RRC  D       0ACB 2 NOP 1
	{2,0x0B,0xCB,0   ,0   ,0}, //RRC  E       0BCB 2 NOP 1
	{2,0x0C,0xCB,0   ,0   ,0}, //RRC  H       0CCB 2 NOP 1
	{2,0x0D,0xCB,0   ,0   ,0}, //RRC  L       0DCB 2 NOP 1
	{1,0x0F,0   ,0   ,0   ,0}, //RRCA ""      0F   1 NOP 1
	{2,0x67,0xED,0   ,0   ,0}, //RRD  ""      67ED 2 NOP 1

	{1,0xC7,0   ,0   ,0   ,0}, //RST  00H     C7   1 NOP 1
	{1,0xCF,0   ,0   ,0   ,0}, //RST  08H     CF   1 NOP 1
	{1,0xD7,0   ,0   ,0   ,0}, //RST  10H     D7   1 NOP 1
	{1,0xDF,0   ,0   ,0   ,0}, //RST  18H     DF   1 NOP 1
	{1,0xE7,0   ,0   ,0   ,0}, //RST  20H     E7   1 NOP 1
	{1,0xEF,0   ,0   ,0   ,0}, //RST  28H     EF   1 NOP 1
	{1,0xF7,0   ,0   ,0   ,0}, //RST  30H     F7   1 NOP 1
	{1,0xFF,0   ,0   ,0   ,0}, //RST  38H     FF   1 NOP 1

	{1,0x9E,0   ,0   ,0   ,0}, //SBC  A,(HL)  9E   1 NOP 1
	{3,0x9E,0xDD,0   ,0   ,0}, //SBC  A,(IX*) 9EDD 3 ZIX 1
	{3,0x9E,0xFD,0   ,0   ,0}, //SBC  A,(IY*) 9EFD 3 ZIX 1
	{1,0x9F,0   ,0   ,0   ,0}, //SBC  A,A     9F   1 NOP 1
	{1,0x98,0   ,0   ,0   ,0}, //SBC  A,B     98   1 NOP 1
	{1,0x99,0   ,0   ,0   ,0}, //SBC  A,C     99   1 NOP 1
	{1,0x9A,0   ,0   ,0   ,0}, //SBC  A,D     9A   1 NOP 1
	{1,0x9B,0   ,0   ,0   ,0}, //SBC  A,E     9B   1 NOP 1
	{1,0x9C,0   ,0   ,0   ,0}, //SBC  A,H     9C   1 NOP 1
	{1,0x9D,0   ,0   ,0   ,0}, //SBC  A,L     9D   1 NOP 1
	{2,0x42,0xED,0   ,0   ,0}, //SBC  HL,BC   42ED 2 NOP 1
	{2,0x52,0xED,0   ,0   ,0}, //SBC  HL,DE   52ED 2 NOP 1
	{2,0x62,0xED,0   ,0   ,0}, //SBC  HL,HL   62ED 2 NOP 1
	{2,0x72,0xED,0   ,0   ,0}, //SBC  HL,SP   72ED 2 NOP 1
	{2,0xDE,0   ,0   ,0   ,0}, //SBC  A,*     DE   2 NOP 1
	{1,0x37,0   ,0   ,0   ,0}, //SCF  ""      37   1 NOP 1

	{2,0xC6,0xCB,0   ,0   ,0}, //SET  *,(HL)  C6CB 2 ZBIT 1
	{4,0xCB,0xDD,0xC6,0x00,0}, //SET  *,(IX*) CBDD 4 ZBIT 1 0 C600
	{4,0xCB,0xFD,0xC6,0x00,0}, //SET  *,(IY*) CBFD 4 ZBIT 1 0 C600
	{2,0xC7,0xCB,0   ,0   ,0}, //SET  *,A     C7CB 2 ZBIT 1
	{2,0xC0,0xCB,0   ,0   ,0}, //SET  *,B     C0CB 2 ZBIT 1
	{2,0xC1,0xCB,0   ,0   ,0}, //SET  *,C     C1CB 2 ZBIT 1
	{2,0xC2,0xCB,0   ,0   ,0}, //SET  *,D     C2CB 2 ZBIT 1
	{2,0xC3,0xCB,0   ,0   ,0}, //SET  *,E     C3CB 2 ZBIT 1
	{2,0xC4,0xCB,0   ,0   ,0}, //SET  *,H     C4CB 2 ZBIT 1
	{2,0xC5,0xCB,0   ,0   ,0}, //SET  *,L     C5CB 2 ZBIT 1

	{2,0x26,0xCB,0   ,0   ,0}, //SLA  (HL)    26CB 2 NOP 1
	{4,0xCB,0xDD,0x26,0x00,0}, //SLA  (IX*)   CBDD 4 ZIX 1 0 2600
	{4,0xCB,0xFD,0x26,0x00,0}, //SLA  (IY*)   CBFD 4 ZIX 1 0 2600
	{2,0x27,0xCB,0   ,0   ,0}, //SLA  A       27CB 2 NOP 1
	{2,0x20,0xCB,0   ,0   ,0}, //SLA  B       20CB 2 NOP 1
	{2,0x21,0xCB,0   ,0   ,0}, //SLA  C       21CB 2 NOP 1
	{2,0x22,0xCB,0   ,0   ,0}, //SLA  D       22CB 2 NOP 1
	{2,0x23,0xCB,0   ,0   ,0}, //SLA  E       23CB 2 NOP 1
	{2,0x24,0xCB,0   ,0   ,0}, //SLA  H       24CB 2 NOP 1
	{2,0x25,0xCB,0   ,0   ,0}, //SLA  L       25CB 2 NOP 1

	{2,0x76,0xED,0   ,0   ,0}, //SLP  ""      76ED 2 NOP 2
	{2,0x2E,0xCB,0   ,0   ,0}, //SRA  (HL)    2ECB 2 NOP 1
	{4,0xCB,0xDD,0x2E,0x00,0}, //SRA  (IX*)   CBDD 4 ZIX 1 0 2E00
	{4,0xCB,0xFD,0x2E,0x00,0}, //SRA  (IY*)   CBFD 4 ZIX 1 0 2E00
	{2,0x2F,0xCB,0   ,0   ,0}, //SRA  A       2FCB 2 NOP 1
	{2,0x28,0xCB,0   ,0   ,0}, //SRA  B       28CB 2 NOP 1
	{2,0x29,0xCB,0   ,0   ,0}, //SRA  C       29CB 2 NOP 1
	{2,0x2A,0xCB,0   ,0   ,0}, //SRA  D       2ACB 2 NOP 1
	{2,0x2B,0xCB,0   ,0   ,0}, //SRA  E       2BCB 2 NOP 1
	{2,0x2C,0xCB,0   ,0   ,0}, //SRA  H       2CCB 2 NOP 1
	{2,0x2D,0xCB,0   ,0   ,0}, //SRA  L       2DCB 2 NOP 1

	{2,0x3E,0xCB,0   ,0   ,0}, //SRL  (HL)    3ECB 2 NOP 1
	{4,0xCB,0xDD,0x3E,0x00,0}, //SRL  (IX*)   CBDD 4 ZIX 1 0 3E00
	{4,0xCB,0xFD,0x3E,0x00,0}, //SRL  (IY*)   CBFD 4 ZIX 1 0 3E00
	{2,0x3F,0xCB,0   ,0   ,0}, //SRL  A       3FCB 2 NOP 1
	{2,0x38,0xCB,0   ,0   ,0}, //SRL  B       38CB 2 NOP 1
	{2,0x39,0xCB,0   ,0   ,0}, //SRL  C       39CB 2 NOP 1
	{2,0x3A,0xCB,0   ,0   ,0}, //SRL  D       3ACB 2 NOP 1
	{2,0x3B,0xCB,0   ,0   ,0}, //SRL  E       3BCB 2 NOP 1
	{2,0x3C,0xCB,0   ,0   ,0}, //SRL  H       3CCB 2 NOP 1
	{2,0x3D,0xCB,0   ,0   ,0}, //SRL  L       3DCB 2 NOP 1

	{1,0x96,0   ,0   ,0   ,0}, //SUB  (HL)    96   1 NOP 1
	{3,0x96,0xDD,0   ,0   ,0}, //SUB  (IX*)   96DD 3 ZIX 1
	{3,0x96,0xFD,0   ,0   ,0}, //SUB  (IY*)   96FD 3 ZIX 1
	{1,0x97,0   ,0   ,0   ,0}, //SUB  A       97   1 NOP 1
	{1,0x90,0   ,0   ,0   ,0}, //SUB  B       90   1 NOP 1
	{1,0x91,0   ,0   ,0   ,0}, //SUB  C       91   1 NOP 1
	{1,0x92,0   ,0   ,0   ,0}, //SUB  D       92   1 NOP 1
	{1,0x93,0   ,0   ,0   ,0}, //SUB  E       93   1 NOP 1
	{1,0x94,0   ,0   ,0   ,0}, //SUB  H       94   1 NOP 1
	{1,0x95,0   ,0   ,0   ,0}, //SUB  L       95   1 NOP 1
	{2,0xD6,0   ,0   ,0   ,0}, //SUB  *       D6   2 NOP 1

	{2,0x3C,0xED,0   ,0   ,0}, //TST  A       3CED 2 NOP 2
	{2,0x04,0xED,0   ,0   ,0}, //TST  B       04ED 2 NOP 2
	{2,0x0C,0xED,0   ,0   ,0}, //TST  C       0CED 2 NOP 2
	{2,0x14,0xED,0   ,0   ,0}, //TST  D       14ED 2 NOP 2
	{2,0x1C,0xED,0   ,0   ,0}, //TST  E       1CED 2 NOP 2
	{2,0x24,0xED,0   ,0   ,0}, //TST  H       24ED 2 NOP 2
	{2,0x2C,0xED,0   ,0   ,0}, //TST  L       2CED 2 NOP 2
	{2,0x34,0xED,0   ,0   ,0}, //TST  (HL)    34ED 2 NOP 2
	{3,0x64,0xED,0   ,0   ,0}, //TST  *       64ED 3 NOP 2

	{2,0x74,0xED,0   ,0   ,0}, //TSTIO *      74ED 3 NOP 2

	{1,0xAE,0   ,0   ,0   ,0}, //XOR  (HL)    AE   1 NOP 1
	{3,0xAE,0xDD,0   ,0   ,0}, //XOR  (IX*)   AEDD 3 ZIX 1
	{3,0xAE,0xFD,0   ,0   ,0}, //XOR  (IY*)   AEFD 3 ZIX 1
	{1,0xAF,0   ,0   ,0   ,0}, //XOR  A       AF   1 NOP 1
	{1,0xA8,0   ,0   ,0   ,0}, //XOR  B       A8   1 NOP 1
	{1,0xA9,0   ,0   ,0   ,0}, //XOR  C       A9   1 NOP 1
	{1,0xAA,0   ,0   ,0   ,0}, //XOR  D       AA   1 NOP 1
	{1,0xAB,0   ,0   ,0   ,0}, //XOR  E       AB   1 NOP 1
	{1,0xAC,0   ,0   ,0   ,0}, //XOR  H       AC   1 NOP 1
	{1,0xAD,0   ,0   ,0   ,0}, //XOR  L       AD   1 NOP 1
	{2,0xEE,0   ,0   ,0   ,0}, //XOR  *       EE   2 NOP 1

	};



char* ChangeExtensionAuFichier(char* NomDuFichier, char* Extension);
void Etape1();
void Etape2();
void Etape3();
int LitInstruction( int AdresseInstruction );
void AdapteAdresse( char *Instruction,int IndexAdresse );
void ModifieAdresse(char *Instruction, int IndexAdresse, int difference);
void AdapteApel( char *Instruction,int IndexAdresse );
void ModifieApel(char *Instruction, int IndexAdresse, unsigned short AdresseDeRemplacement);

int main(int argc, char* argv[])
{
	int i;
	printf("ION83EMUPC par Marc Plouhinec\n");

	if(argc == 1)
	{
		printf( "syntaxe: ION83EMUPC -FICHIER.83P <-debug>\n");
		return 0;
	}

	if (argc == 3)
		DebugMode = true;

	//On ouvre le fichier .83p
	_fmode=_O_BINARY;
	if (!(fichier_83p = fopen(argv[1], "r")))
	{	
		fprintf(stderr, "Erreur: Impossible de lire: %s\n", argv[1]);
		return 2;
	}
	fgetpos(fichier_83p,&pos);
	//On crer le fichier .82p
	if (!(fichier_82p = fopen(ChangeExtensionAuFichier(argv[1],"82P"), "wb")))
	{	
		fprintf(stderr, "Erreur: Impossible de creer: %s\n", ChangeExtensionAuFichier(argv[1],"82P"));
		return 2;
	}
	//On saute vers le code
	for (i = 0; i<0x46; i++) fgetc(fichier_83p);
	//On rcupre la taille du programme
	TailleProgrammeASM = fgetc(fichier_83p);
	TailleProgrammeASM += fgetc(fichier_83p)*0x100;
	//On cherche la premire instruction du programme
	fgetc(fichier_83p);
	fgetc(fichier_83p);
	AdresseRelativePremInstr = 3;
	AdresseRelativePremInstr += fgetc(fichier_83p);
	for (i = 0; i<AdresseRelativePremInstr-3; i++) fgetc(fichier_83p);
	//On charge le programme dans la tableau Programme
	for (i =0; i<TailleProgrammeASM ; i++)
		Programme[i]=fgetc(fichier_83p);
	//Etape 1
	printf("\n1-Separation Donnee/Code\n");
	Etape1();


	//test
	if(DebugMode)
	{
		FILE* fichier_test;
		if (!(fichier_test = fopen("TEST.TXT", "wb")))
		{	
			fprintf(stderr, "Erreur: Impossible de creer: %s\n", "TEST.TXT");
			return 3;
		}
		for( i =0; i<TailleProgrammeASM; i++)
			fputc(OmbreDuProgramme[i],fichier_test);
		if (!(fichier_test = fopen("TEST2.TXT", "wb")))
		{	
			fprintf(stderr, "Erreur: Impossible de creer: %s\n", "TEST2.TXT");
			return 3;
		}
		for( i =0; i<TailleProgrammeASM; i++)
			fputc(Programme[i],fichier_test);
	}


	//Etape 2
	printf("2-Adaptation du code pour la TI-82\n");
	Etape2();

	//Etape 3
	printf("3-Creation du fichier .82p\n");
	Etape3();

	//Fini!
	printf("\nAdaptation terminee, utilisez le programme ION83EMU pour lancer votre programme\n\n");





	return 0;
}


char* ChangeExtensionAuFichier(char* NomDuFichier, char* Extension)
{
	int i=0;
	while(*NomDuFichier != '.')
	{
		Nom[i] = *NomDuFichier;
		NomDuFichier++;
		i++;
	}
	Nom[i]='.';
	for( int j = 1; j<4; j++)
	{
		Nom[i+j] = *Extension;
		Extension++;
	}
	Nom[i+j] = 0;
	return Nom;
}

//1-dterminer ce qui est donne et ce qui est instruction en stockant les liens (sauts, appels, LD,...)
//  vers la rom et la ram.
void Etape1()
{
/*Mode opratoire:
On lis le code en placant des 1 dans le tableau au niveau des instructions lues
sauf si c'est des liens vers des adresses (JP, CALL, LD,...)
Quand on arrive  un saut ou un appel on place un 1 vers l'adresse vise seulement si l'adresse
fais partie du programme.
A chaque 1 on lit l'instruction suivante
On recommence ces opration TailleProgrammeASM fois pour tre sr d'avoir tout vu.
OmbreDuProgramme contiendra ainsi tout ce que nous devons savoir.
*/
	//On fait une barre de chargement
	int ChargementActuel = 0;
	printf("\n0-----------------------50---------------------100\n");

	LitInstruction(0);

	for(int i = 0; i<= TailleProgrammeASM; i++)
	{
		for( int j = 0; j < TailleProgrammeASM; j++)
		{
			if(OmbreDuProgramme[j]!=0)
			{
				j=LitInstruction(j)-1;
			}
		}
		//On donne une indication du calcul restant:
		if( i*50/TailleProgrammeASM != ChargementActuel)
		{
			ChargementActuel=i*50/TailleProgrammeASM;
			printf("_");
		}
	}
	printf("\n\n");
}


int LitInstruction( int AdresseInstruction )
{
/*
On lit l'instruction pointe par AdresseInstruction,
-si c'est une instruction quelcqonque on place des 1  la place des donnes occupes par l'instruction
et on place un 1 juste aprs l'instruction.
-si c'est un RET on remplis l'espace occup par des 1 mais on ne place pas un 1 juste aprs
-si c'est un JP on remplis l'espace occup par des 1 et on met un 1  l'adresse pointe par l'adresse
 (si elle n'appartient pas au programme on remplis OmbreDuProgramme directement par l'instruction),
 si c'est JP (HL) on remplis juste l'espace occup par des 1
-si c'est un saut relatif (JR, DJNZ) on fais exactement la mme chose qu'avec JP
-si c'est un CALL on fais exactement la mme chose qu'avec JP sauf qu'on met un 1  l'instruction suivante
-si c'est un LD qui utilise une adresse de 16 bits on ne met pas de 1 met on copie l'instruction


IMPORTANT! si l'instruction suivante n'est pas un 0 on ne met rien dans OmbreDuProgramme.
Autre prcaution, on sauvegarde l'adresse des LD BC,? car son opcode est 1
*/
	char Instruction[4];
	int i,j;
	unsigned short AdressePointee;
	char AdresseRelative;
	for(i = 0; i < 4; i++)
		Instruction[i] = Programme[AdresseInstruction+i];
	//On cherche d'abord sur combien d'octets cette instruction est dfinie (sans compter les donnes associes)
	if (Instruction[0] == (char)(0xED) || Instruction[0] == (char)(0xCB) || Instruction[0] == (char)(0xDD) || Instruction[0] == (char)(0xFD))
	{
		//Cette instruction est dfinie sur au moins 2 octets
		if(Instruction[1] == (char)(0xCB))
		{///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			//Cette instruction est dfinie sur 4 octets
			//Ce cas est est simplicime, il n'y a aucun cas "pathologique"!
			if(OmbreDuProgramme[AdresseInstruction+1] == 0)
			{
				for(i = 0; i<=4; i++)
					OmbreDuProgramme[AdresseInstruction+i] = 1;
			}
			return AdresseInstruction+4;
		}
		else///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		{
			//Cette instruction est dfinie sur 2 octets
			for(i = 0; i < 519 ; i++)
				if (Instruction[1] == (char)RefInstructions[i][1] && Instruction[0] == (char)RefInstructions[i][2] && (char)RefInstructions[i][2]!=0)
					//S'il n'y a pas de 0 juste aprs c'est que l'instruction a dj t traite.
					if(OmbreDuProgramme[AdresseInstruction+1] == 0)
					{
						//On a reconnu l'instruction, est-elle spciale? (0 = rien; 4=LD+16bits; 5=RET)
						if (RefInstructions[i][5] == 0)
						{
							//non
							for( j=0;j <= RefInstructions[i][0]; j++)       //
								OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction et un 1 juste aprs
							return AdresseInstruction+RefInstructions[i][0];
						}
						if (RefInstructions[i][5] == 4)
						{
							//c'est un LD+16bits
							for( j=0;j < RefInstructions[i][0]; j++)                     //
								OmbreDuProgramme[AdresseInstruction+j] = Instruction[j]; //On recopie l'instruction
							OmbreDuProgramme[AdresseInstruction+RefInstructions[i][0]] = 1; //On place un 1 juste aprs
							return AdresseInstruction+RefInstructions[i][0];
						}
						if (RefInstructions[i][5] == 5)
						{
							//c'est un RET
							for( j=0;j < RefInstructions[i][0]; j++)        //
								OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction
							if (Instruction[1]!=(char)0x4D && Instruction[1]!=(char)0x45)
							{
								OmbreDuProgramme[AdresseInstruction+j] = 1;
								return AdresseInstruction+RefInstructions[i][0];
							}
							//test: printf("ret adresse:%x\ninstruction %x %x %x %x",AdresseInstruction, Instruction[0], Instruction[1], Instruction[2], Instruction[3]);
							return AdresseInstruction+RefInstructions[i][0];
						}
					}
					else
						return AdresseInstruction+RefInstructions[i][0];
		//On a pas reconnu l'instruction, c'est un ZBIT qui prend donc 2 octets
					for( j=0;j <= 2; j++)        //
						OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction
					return AdresseInstruction+2;
		//On a pas reconnu l'instruction, c'est qu'il y a eu un problme, on doit arrter l
		//	printf("ERREUR instruction %x %x %x %x non reconnue!\n", Instruction[0], Instruction[1], Instruction[2], Instruction[3]);
		//	return TailleProgrammeASM;
		}
	}
	else///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	{
		//Cette instruction est dfinie sur 1 octet
		for(i = 0; i < 519 ; i++)
		{
			if (Instruction[0] == (char)RefInstructions[i][1] && (char)RefInstructions[i][2]==0)
			{
				//S'il n'y a pas de 0 juste aprs c'est que l'instruction a dj t traite.
				if(OmbreDuProgramme[AdresseInstruction+1] == 0)
				{
					//On a reconnu l'instruction, est-elle spciale? (0 = rien; 1 = JP; 2 = JR ou DJNZ; 3=CALL; 4=LD+16bits; 5=RET)
					if (RefInstructions[i][5] == 0)
					{
						//non
						for( j=0;j <= RefInstructions[i][0]; j++)       //
							OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction et un 1 juste aprs
						return AdresseInstruction+RefInstructions[i][0];
					}
					if (RefInstructions[i][5] == 1)
					{
						//c'est un JP
						for( j=0;j < RefInstructions[i][0]; j++)        //
							OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction
						if (Instruction[0]!=(char)0xC3 && OmbreDuProgramme[AdresseInstruction+j] == 0) OmbreDuProgramme[AdresseInstruction+j] = 1;
						//Il faut maintenant voir si l'adresse pointe par JP est dans le programme
						AdressePointee = (unsigned char)Instruction[1]+Instruction[2]*0x100;
						if(AdressePointee < (unsigned short)PROGSTART || AdressePointee > (unsigned short)(PROGSTART+TailleProgrammeASM))
						{
							//c'est un saut vers une instruction qui n'appartient pas au programme
							//c'est un cas spcial, on remplace les 1 par l'instruction.
							for( j=0;j < RefInstructions[i][0]; j++)
								OmbreDuProgramme[AdresseInstruction+j] = Instruction[j];
						}
						else
						{
							//c'est un saut vers le programme, on place un 1  l'adresse pointe
							if(OmbreDuProgramme[(unsigned short)(AdressePointee-PROGSTART-AdresseRelativePremInstr)] == 0)
								OmbreDuProgramme[(unsigned short)(AdressePointee-PROGSTART-AdresseRelativePremInstr)] = 1;
						}
						return AdresseInstruction+RefInstructions[i][0];
					}
					if (RefInstructions[i][5] == 2)
					{
						//c'est un JR ou un DJNZ
						for( j=0;j < RefInstructions[i][0]; j++)        //
							OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction
						if (Instruction[0]!=(char)0x18 && OmbreDuProgramme[AdresseInstruction+j] == 0) OmbreDuProgramme[AdresseInstruction+j] = 1;
						AdresseRelative = Instruction[1]+2;
						//on place un 1  l'adresse pointe
						if(OmbreDuProgramme[AdresseRelative+AdresseInstruction] ==0)
							OmbreDuProgramme[AdresseRelative+AdresseInstruction] = 1;
						return AdresseInstruction+RefInstructions[i][0];
					}
					if (RefInstructions[i][5] == 3)
					{
						//c'est un CALL
						for( j=0;j <= RefInstructions[i][0]; j++)        //
							OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction et juste aprs
						//Il faut maintenant voir si l'adresse pointe par CALL est dans le programme
						AdressePointee = (unsigned char)Instruction[1]+Instruction[2]*0x100;
						if(AdressePointee < (unsigned short)(PROGSTART) || AdressePointee > (unsigned short)(PROGSTART+TailleProgrammeASM))
						{
							//c'est un appel vers une instruction qui n'appartient pas au programme
							//c'est un cas spcial, on remplace les 1 par l'instruction.
							for( j=0;j < RefInstructions[i][0]; j++)
								OmbreDuProgramme[AdresseInstruction+j] = Instruction[j];
						}
						else
						{
							//c'est un appel vers le programme, on place un 1  l'adresse pointe
							if (OmbreDuProgramme[(unsigned short)(AdressePointee-PROGSTART-AdresseRelativePremInstr)] == 0)
								OmbreDuProgramme[(unsigned short)(AdressePointee-PROGSTART-AdresseRelativePremInstr)] = 1;
						}
						return AdresseInstruction+RefInstructions[i][0];
					}
					if (RefInstructions[i][5] == 4)
					{
						//c'est un LD+16bits
						for( j=0;j < RefInstructions[i][0]; j++)                     //
							OmbreDuProgramme[AdresseInstruction+j] = Instruction[j]; //On recopie l'instruction
						if (OmbreDuProgramme[AdresseInstruction+RefInstructions[i][0]] ==0)
							OmbreDuProgramme[AdresseInstruction+RefInstructions[i][0]] = 1; //On place un 1 juste aprs
						if(Instruction[0]==0x01) //c'est un LD BC,?
						{
							AdresseLDBC[AdresseInstruction]=1;
						}
						return AdresseInstruction+RefInstructions[i][0];
					}
					if (RefInstructions[i][5] == 5)
					{
						//c'est un RET
						for( j=0;j < RefInstructions[i][0]; j++)       //
							OmbreDuProgramme[AdresseInstruction+j] = 1; //On place des 1 sur l'instruction
						if (Instruction[0]!=(char)0xC9)
						{
							OmbreDuProgramme[AdresseInstruction+j] = 1;
							return AdresseInstruction+RefInstructions[i][0];
						}
						//test: printf("ret adresse:%x\ninstruction %x %x %x %x",AdresseInstruction, Instruction[0], Instruction[1], Instruction[2], Instruction[3]);
						return AdresseInstruction+RefInstructions[i][0];
					}
				}
				else
					return AdresseInstruction+RefInstructions[i][0];
			}
		}
		//On a pas reconnu l'instruction, c'est qu'il y a eu un problme, on doit arrter l
		printf("ERREUR instruction %x %x %x %x non reconnue!\n", Instruction[0], Instruction[1], Instruction[2], Instruction[3]);
		return TailleProgrammeASM;
	}

	return TailleProgrammeASM;
}


/////////////////////////////////////////////ETAPE 2///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////



void Etape2()//On adapte les appels vers la rom et les liens vers les buffers de la ram
{
	int i,j;
	char Instruction[4];
	for( i = 0 ; i < TailleProgrammeASM; i++)
	{
		if((OmbreDuProgramme[i] != 0 && OmbreDuProgramme[i] !=1) || AdresseLDBC[i]==1)
		{
			for(j = 0; j < 4; j++)
				Instruction[j] = OmbreDuProgramme[i+j];
			//On cherche l'instruction
			if (Instruction[0] == (char)(0xED) || Instruction[0] == (char)(0xCB) || Instruction[0] == (char)(0xDD) || Instruction[0] == (char)(0xFD))
			{
				//Cette instruction est dfinie sur 2 octets, il ne reste que les LD+16bits
				AdapteAdresse(Instruction,2);
				for(j = 0; j < 4; j++)
					Programme[i+j]=Instruction[j];
				i+=3;
			}
			else
			{
				//Cette instruction est dfinie sur 1 octets
				for(j = 0; j < 519 ; j++)
				{
					if (Instruction[0] == (char)RefInstructions[j][1] && (char)RefInstructions[j][2]==0)
					{
						//est-ce un LD?
						if(RefInstructions[j][5] == 4)
							AdapteAdresse(Instruction,1);
						//est-ce un CALL?
						if(RefInstructions[j][5] == 3)
							AdapteApel(Instruction,1);
						//est-ce un JP?
						if(RefInstructions[j][5] == 1)
							AdapteApel(Instruction,1);
						j = 519;
					}
				}
				for(j = 0; j < 3; j++)
					Programme[i+j]=Instruction[j];
				i+=2;
			}	
		}
	}

}








void AdapteAdresse( char *Instruction,int IndexAdresse )
{
/*
contrast	=$8008
currow		=800Ch
curcol		=800Dh ->on touche pas
saferam4	=8039h ->OP1 ($8028)
op1		=8039h
op2		=8044h
op3		=804Fh
op4		=805Ah
op5		=8065h
op6		=8070h
imathptr1	=8094h ->EMUBUFFER1 ($8D26), 10 octets
imathptr2	=8096h
imathptr3	=8098h
imathptr4	=809Ah
imathptr5	=809Ch
saferam3	=80C9h ->TEXT_MEM ($808F), 128 octets
pencol		=8252h ->CURSOR_X ($8215)
penrow		=8253h ->CURSOR_Y ($8216)
saferam1	=8265h ->APD_BUF  ($8228), 768 octets
usermemoff	=$8565 ->EMUBUFFER2 ($8D30), 42 octets
saferam2	=858Fh ->EMUBUFFER3 ($8D5A), 531 octets
curgy		=886Dh ->EMUBUFFER4 ($8F6D), 1 octet
curgx		=886Eh ->EMUBUFFER5 ($8F6E), 1 octet
plotsscreen	=8E29h ->GRAPH_MEM ($88B8), 768 octets
cmdshad		=9157h ->TEXT_MEM2 ($8BDF),128 octets
ptemp		=9317h ->$8D18
progptr		=9319h ->EMUBUFFER6 ($8F6F), 2 octets contenant $FD9D
progstart	=9327h

587 octets doivent tre crs au dbut de SNG

----explication de buffers
call _grphCirc
(curgx) >> coordonne x d'un point du cercle
(curgy) >> coordonne y d'un point du cercle
(curgx2) >> coordonne x du centre du cercle
(curgy2) >> coordonne y du centre du cercle
*/
	unsigned short AdresseVisee;
	AdresseVisee = (unsigned char)Instruction[IndexAdresse]+Instruction[IndexAdresse+1]*0x100;
	//<0x8039 ;on ne touche pas
	if(AdresseVisee < (unsigned short)0x8039) return;
	//0x8039>= ? <0x8094 on rammenne  0x8028
	if(AdresseVisee>=0x8039 && AdresseVisee<0x8094)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8039-0x8028);
	}
	//0x8094<= ? <0x809C+2 on rammenne  0x8D26
	if(AdresseVisee>=0x8094 && AdresseVisee<0x809C+2)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8094-0x8D26);
	}
	//0x80C9 <= ? <0x80C9+128 on ramenne  0x808F
	if(AdresseVisee>=0x80C9 && AdresseVisee<0x80C9+128)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x80C9-0x808F);
	}
	//0x8252 <= ? <0x8253+1 on rammenne  0x8215
	if(AdresseVisee>=0x8252 && AdresseVisee<0x8253+1)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8252-0x8215);
	}
	//0x8265 <= ? <0x8265+768 on ramenne  0x8228
	if(AdresseVisee>=0x8265 && AdresseVisee<0x8265+768)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8265-0x8228);
	}
	//0x8565 <= ? <0x8565+42 on rammene  0x8D30
	if(AdresseVisee>=0x8565 && AdresseVisee<0x8565+42)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8565-0x8D30);
	}
	//0x858F <= ? <0x858F+531 on rammene  0x8D5A
	if(AdresseVisee>=0x858F && AdresseVisee<0x858F+531)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x858F-0x8D5A);
	}
	//0x886D <= ? <0x886E+1 on rammene  0x8F6D
	if(AdresseVisee>=0x886D && AdresseVisee<0x886E+1)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x886D-0x8F6D);
	}
	//0x8E29 <= ? <0x9157 on rammene  0x88B8
	if(AdresseVisee>=0x8E28 && AdresseVisee<0x9157)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x8E29-0x88B8);
	}
	//0x9157 <= ? <0x9157+128 on rammene  0x8BDF
	if(AdresseVisee>=0x9157 && AdresseVisee<0x9157+128)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x9157-0x8BDF);
	}
	//0x9317 <= ? <0x9317+2 on rammene  0x8D18
	if(AdresseVisee>=0x9317 && AdresseVisee<0x9317+2)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x9317-0x8D18);
	}
	//0x9319 <= ? <0x9319+2 on rammene  0x8F6F
	if(AdresseVisee>=0x9319 && AdresseVisee<0x9319+2)
	{
		ModifieAdresse(Instruction, IndexAdresse, 0x9319-0x8F6F);
	}
}

void ModifieAdresse(char *Instruction, int IndexAdresse, int difference)
{
	unsigned short AdresseVisee;
	AdresseVisee = (unsigned char)Instruction[IndexAdresse]+Instruction[IndexAdresse+1]*0x100;
	Instruction[IndexAdresse+1] = (unsigned short)(AdresseVisee-(difference))/0x100;
	Instruction[IndexAdresse  ] = (unsigned short)(AdresseVisee-(difference))-(unsigned char)(Instruction[IndexAdresse+1])*0x100;
}





void AdapteApel( char *Instruction,int IndexAdresse )
{
#define TableRomEMU 0x8F71
/*
Principe: on redirige tout les appels vers la rom donn par ion.inc
vers une table de sauts  l'adresse 0x8F71
;-----> Ion Functions
lVectors	=cmdshad+80       = $9157+80=$91A7 = TableRomEMU
ionVersion	=lVectors         =$91A7 = TableRomEMU
ionRandom	=lVectors+3       =$91AA = TableRomEMU+3
ionPutSprite	=lVectors+6   =$91AD = TableRomEMU+6
ionLargeSprite	=lVectors+9   =$91B0 = TableRomEMU+9
ionGetPixel	=lVectors+12      =$91B3 = TableRomEMU+12
ionFastCopy	=lVectors+15      =$91B6 = TableRomEMU+15
ionDetect	=lVectors+18      =$91B9 = TableRomEMU+18
ionDecompress	=lVectors+21  =$91BC = TableRomEMU+21


_ldhlind	=4000h = TableRomEMU+24
_cphlde		=4004h = TableRomEMU+27
_divhlby10	=4008h = TableRomEMU+30
_divhlbya	=400Ch = TableRomEMU+33
_getcsc		=$4014 = TableRomEMU+36
_lcdbusy	=4066h = TableRomEMU+39
_hltimes9	=4146h = TableRomEMU+42
_mov9toop1	=$4206 = TableRomEMU+45
_zeroop1	=428Eh = TableRomEMU+48
_htimesl	=$4382 = TableRomEMU+51
_memfree	=441Ah = TableRomEMU+54
_chksysvar	=442Ah = TableRomEMU+57
_findsym	=442Eh = TableRomEMU+60
_insertmem	=4432h = TableRomEMU+63
_createreal	=4452h = TableRomEMU+66
_creategdb	=4486h = TableRomEMU+69
_createprog	=448Ah = TableRomEMU+72
_delvar		=44AAh = TableRomEMU+75
_deletemem	=44B2h = TableRomEMU+78
_putmap		=4701h = TableRomEMU+81
_putc		=4705h = TableRomEMU+84
_disphl		=4709h = TableRomEMU+87
_puts		=470Dh = TableRomEMU+90
_putps		=4715h = TableRomEMU+93
_newline	=$473D = TableRomEMU+96
_clrlcdf	=4755h = TableRomEMU+99
_clrscr		=4759h = TableRomEMU+102
_clrscrf	=475Dh = TableRomEMU+105
_cleartextshad	=4765h = TableRomEMU+108
_homeup		=$4775 = TableRomEMU+111
_vputmap	=477Dh = TableRomEMU+114
_vputs		=4781h = TableRomEMU+117
_vputsn		=4785h = TableRomEMU+120
_indicatorOn	=$4791 = TableRomEMU+123
_indicatorOff	=$4795 = TableRomEMU+126
_getk		=4A18h = TableRomEMU+129
_FPSUB      =408Eh = TableRomEMU+132
_FPDIV      =40C6h = TableRomEMU+135
_FPMULT     =40AAh = TableRomEMU+138
_CONVOP1    =4EFCh = TableRomEMU+141
_copygbuf	=4B9Ch = TableRomEMU+144
_getkey		=4CFEh = TableRomEMU+147
_setxxxxop2	=$4A7C = TableRomEMU+150
_setxxop1	=$4A74 = TableRomEMU+153
_cleargbuf	=515Bh = TableRomEMU+159
_dispop1a	=$51D4 = TableRomEMU+162
_op2toop1	=$41C2 = TableRomEMU+165
_SFONT_LEN  =$4A6C = TableRomEMU+180



Ces routines ne seront pas mules
_createstring	=4472h
_parseinp   =$4E8C
_rcly		=4EE0h
_rclx		=4EE8h
_cline		=4A84h
_darkline	=$4AE0
_iline		=4AE4h
_ipoint		=4AE8h

_op1toop2	=$418E
_op1toop3	=$417E
_op1toop4	=$416E
_op1toop5	=$41BE
_op1toop6	=$41BA
_op2toop3	=$41F6
_op2toop4	=$4172
_op2toop5	=$41B2
_op2toop6	=$41B6
_op3toop1	=$41A2
_op3toop2	=$417A
_op3toop4	=$416A
_op3toop5	=$41AE
_op4toop1	=$419A
_op4toop2	=$4176
_op4toop3	=$41FA
_op4toop5	=$41AA
_op4toop6	=$4202
_op5toop1	=$419E
_op5toop2	=$4182
_op5toop3	=$41FE
_op5toop4	=$418A
_op5toop6	=$4186
_op6toop1	=$4196
_op6toop2	=$4192
_op6toop5	=$41A6
_op1exop2	=$4306
_op1exop3	=$42FE
_op1exop4	=$4302
_op1exop5	=$42EE
_op1exop6	=$42F2
_op2exop4	=$42F6
_op2exop5	=$42FA
_op2exop6	=$42E6
_op5exop6	=$42EA
_PUSHREALO1        .EQU           4536h
_PUSHREALO4        .EQU           452Ah
_POPREALO2         .EQU           44FAh
_POPREALO5         .EQU           44EEh
_PUSHREALO2        .EQU           4532h
_PUSHREALO5        .EQU           4526h
_POPREALO3         .EQU           44F6h
_POPREALO6         .EQU           44EAh
_PUSHREALO3        .EQU           452Eh
_PUSHREALO6        .EQU           4522h
_POPREALO1         .EQU           44FEh
_POPREALO4         .EQU           44F2h
_FPMULT            .EQU           40AAh
_FPADD             .EQU           4092h
_OP1SET0           .EQU           4286h
_SIN               .EQU           40F6h
_op1set1	=$4232
_op1set2	=$4242
_op1set3	=$423A
_op1set4	=$4236
_op2set0	=$4282
_op2set1	=$4252
_op2set2	=$4246
_op2set3	=$422E
_op2set4	=$422A
_op2set5	=$4222
_op2set8	=$421E
_op2set60	=$4EA8
_op2setA	=$4226
_op3set0	=$427E
_op3set1	=$421A
_op3set2	=$423E
_op4set0	=$427A
_op4set1	=$4216
_op5set0	=$4276
_pushop1	=$454A
_pushop3	=$4542
_popop1		=$44E6
_popop3		=$44E2
_setxxop2	=$4A78



*/
	AdresseTrouvee = false;
	unsigned short AdresseVisee;
	AdresseVisee = (unsigned char)Instruction[IndexAdresse]+Instruction[IndexAdresse+1]*0x100;

//$003A = $0038
	if(AdresseVisee == 0x003A) ModifieApel(Instruction,IndexAdresse,0x0038);
//ionVersion	=lVectors         =$91A7 = TableRomEMU
	if(AdresseVisee == 0x91A7) ModifieApel(Instruction,IndexAdresse,TableRomEMU);
//ionRandom	=lVectors+3       =$91AA = TableRomEMU+3
	if(AdresseVisee == 0x91AA) ModifieApel(Instruction,IndexAdresse,TableRomEMU+3);
//ionPutSprite	=lVectors+6   =$91AD = TableRomEMU+6
	if(AdresseVisee == 0x91AD) ModifieApel(Instruction,IndexAdresse,TableRomEMU+6);
//ionLargeSprite	=lVectors+9   =$91B0 = TableRomEMU+9
	if(AdresseVisee == 0x91B0) ModifieApel(Instruction,IndexAdresse,TableRomEMU+9);
//ionGetPixel	=lVectors+12      =$91B3 = TableRomEMU+12
	if(AdresseVisee == 0x91B3) ModifieApel(Instruction,IndexAdresse,TableRomEMU+12);
//ionFastCopy	=lVectors+15      =$91B6 = TableRomEMU+15
	if(AdresseVisee == 0x91B6) ModifieApel(Instruction,IndexAdresse,TableRomEMU+15);
//ionDetect	=lVectors+18      =$91B9 = TableRomEMU+18
	if(AdresseVisee == 0x91B9) ModifieApel(Instruction,IndexAdresse,TableRomEMU+18);
//ionDecompress	=lVectors+21  =$91BC = TableRomEMU+21
	if(AdresseVisee == 0x91BC) ModifieApel(Instruction,IndexAdresse,TableRomEMU+21);


//_ldhlind	=4000h = TableRomEMU+24
	if(AdresseVisee == 0x4000) ModifieApel(Instruction,IndexAdresse,TableRomEMU+24);
//_cphlde		=4004h = TableRomEMU+27
	if(AdresseVisee == 0x4004) ModifieApel(Instruction,IndexAdresse,TableRomEMU+27);
//_divhlby10	=4008h = TableRomEMU+30
	if(AdresseVisee == 0x4008) ModifieApel(Instruction,IndexAdresse,TableRomEMU+30);
//_divhlbya	=400Ch = TableRomEMU+33
	if(AdresseVisee == 0x400C) ModifieApel(Instruction,IndexAdresse,TableRomEMU+33);
//_getcsc		=$4014 = TableRomEMU+36
	if(AdresseVisee == 0x4014) ModifieApel(Instruction,IndexAdresse,TableRomEMU+36);
//_lcdbusy	=4066h = TableRomEMU+39
	if(AdresseVisee == 0x4066) ModifieApel(Instruction,IndexAdresse,TableRomEMU+39);
//_hltimes9	=4146h = TableRomEMU+42
	if(AdresseVisee == 0x4146) ModifieApel(Instruction,IndexAdresse,TableRomEMU+42);
//_mov9toop1	=$4206 = TableRomEMU+45
	if(AdresseVisee == 0x4206) ModifieApel(Instruction,IndexAdresse,TableRomEMU+45);
//_zeroop1	=428Eh = TableRomEMU+48
	if(AdresseVisee == 0x428E) ModifieApel(Instruction,IndexAdresse,TableRomEMU+48);
//_htimesl	=$4382 = TableRomEMU+51
	if(AdresseVisee == 0x4382) ModifieApel(Instruction,IndexAdresse,TableRomEMU+51);
//_memfree	=441Ah = TableRomEMU+54
	if(AdresseVisee == 0x441A) ModifieApel(Instruction,IndexAdresse,TableRomEMU+54);
//_chksysvar	=442Ah = TableRomEMU+57
	if(AdresseVisee == 0x442A) ModifieApel(Instruction,IndexAdresse,TableRomEMU+57);
//_findsym	=442Eh = TableRomEMU+60
	if(AdresseVisee == 0x442E) ModifieApel(Instruction,IndexAdresse,TableRomEMU+60);
//_insertmem	=4432h = TableRomEMU+63
	if(AdresseVisee == 0x4432) ModifieApel(Instruction,IndexAdresse,TableRomEMU+63);
//_createreal	=4452h = TableRomEMU+66
	if(AdresseVisee == 0x4452) ModifieApel(Instruction,IndexAdresse,TableRomEMU+66);
//_creategdb	=4486h = TableRomEMU+69
	if(AdresseVisee == 0x4486) ModifieApel(Instruction,IndexAdresse,TableRomEMU+69);
//_createprog	=448Ah = TableRomEMU+72
	if(AdresseVisee == 0x448A) ModifieApel(Instruction,IndexAdresse,TableRomEMU+72);
//_delvar		=44AAh = TableRomEMU+75
	if(AdresseVisee == 0x44AA) ModifieApel(Instruction,IndexAdresse,TableRomEMU+75);
//_deletemem	=44B2h = TableRomEMU+78
	if(AdresseVisee == 0x44B2) ModifieApel(Instruction,IndexAdresse,TableRomEMU+78);
//_putmap		=4701h = TableRomEMU+81
	if(AdresseVisee == 0x4701) ModifieApel(Instruction,IndexAdresse,TableRomEMU+81);
//_putc		=4705h = TableRomEMU+84
	if(AdresseVisee == 0x4705) ModifieApel(Instruction,IndexAdresse,TableRomEMU+84);
//_disphl		=4709h = TableRomEMU+87
	if(AdresseVisee == 0x4709) ModifieApel(Instruction,IndexAdresse,TableRomEMU+87);
//_puts		=470Dh = TableRomEMU+90
	if(AdresseVisee == 0x470d) ModifieApel(Instruction,IndexAdresse,TableRomEMU+90);
//_putps		=4715h = TableRomEMU+93
	if(AdresseVisee == 0x4715) ModifieApel(Instruction,IndexAdresse,TableRomEMU+93);
//_newline	=$473D = TableRomEMU+96
	if(AdresseVisee == 0x473D) ModifieApel(Instruction,IndexAdresse,TableRomEMU+96);
//_clrlcdf	=4755h = TableRomEMU+99
	if(AdresseVisee == 0x4755) ModifieApel(Instruction,IndexAdresse,TableRomEMU+99);
//_clrscr		=4759h = TableRomEMU+102
	if(AdresseVisee == 0x4759) ModifieApel(Instruction,IndexAdresse,TableRomEMU+102);
//_clrscrf	=475Dh = TableRomEMU+105
	if(AdresseVisee == 0x475D) ModifieApel(Instruction,IndexAdresse,TableRomEMU+105);
//_cleartextshad	=4765h = TableRomEMU+108
	if(AdresseVisee == 0x4765) ModifieApel(Instruction,IndexAdresse,TableRomEMU+108);
//_homeup		=$4775 = TableRomEMU+111
	if(AdresseVisee == 0x4775) ModifieApel(Instruction,IndexAdresse,TableRomEMU+111);
//_vputmap	=477Dh = TableRomEMU+114
	if(AdresseVisee == 0x477D) ModifieApel(Instruction,IndexAdresse,TableRomEMU+114);
//_vputs		=4781h = TableRomEMU+117
	if(AdresseVisee == 0x4781) ModifieApel(Instruction,IndexAdresse,TableRomEMU+117);
//_vputsn		=4785h = TableRomEMU+120
	if(AdresseVisee == 0x4785) ModifieApel(Instruction,IndexAdresse,TableRomEMU+120);
//_indicatorOn	=$4791 = TableRomEMU+123
	if(AdresseVisee == 0x4791) ModifieApel(Instruction,IndexAdresse,TableRomEMU+123);
//_indicatorOff	=$4795 = TableRomEMU+126
	if(AdresseVisee == 0x4795) ModifieApel(Instruction,IndexAdresse,TableRomEMU+126);
//_getk		=4A18h = TableRomEMU+129
	if(AdresseVisee == 0x4A18) ModifieApel(Instruction,IndexAdresse,TableRomEMU+129);
//_FPSUB      =408Eh = TableRomEMU+132
	if(AdresseVisee == 0x408E) ModifieApel(Instruction,IndexAdresse,TableRomEMU+132);
//_FPDIV      =40C6h = TableRomEMU+135
	if(AdresseVisee == 0x40C6) ModifieApel(Instruction,IndexAdresse,TableRomEMU+135);
//_FPMULT     =40AAh = TableRomEMU+138
	if(AdresseVisee == 0x40AA) ModifieApel(Instruction,IndexAdresse,TableRomEMU+138);
//_CONVOP1    =4EFCh = TableRomEMU+141
	if(AdresseVisee == 0x4EFC) ModifieApel(Instruction,IndexAdresse,TableRomEMU+141);
//_copygbuf	=4B9Ch = TableRomEMU+144
	if(AdresseVisee == 0x4B9C) ModifieApel(Instruction,IndexAdresse,TableRomEMU+144);
//_getkey		=4CFEh = TableRomEMU+147
	if(AdresseVisee == 0x4CFE) ModifieApel(Instruction,IndexAdresse,TableRomEMU+147);
//_setxxxxop2	=$4A7C = TableRomEMU+150
	if(AdresseVisee == 0x4A7C) ModifieApel(Instruction,IndexAdresse,TableRomEMU+150);
//_setxxop1	=$4A74 = TableRomEMU+153
	if(AdresseVisee == 0x4A74) ModifieApel(Instruction,IndexAdresse,TableRomEMU+153);
//_cleargbuf	=515Bh = TableRomEMU+159
	if(AdresseVisee == 0x515B) ModifieApel(Instruction,IndexAdresse,TableRomEMU+159);
//_dispop1a	=$51D4 = TableRomEMU+162
	if(AdresseVisee == 0x51D4) ModifieApel(Instruction,IndexAdresse,TableRomEMU+162);
//_op2toop1	=$41C2 = TableRomEMU+165
	if(AdresseVisee == 0x41C2) ModifieApel(Instruction,IndexAdresse,TableRomEMU+165);
//_SFONT_LEN  =$4A6C = TableRomEMU+180
	if(AdresseVisee == 0x4A6C) ModifieApel(Instruction,IndexAdresse,TableRomEMU+180);

	if(!AdresseTrouvee)
	{
		printf("L'adresse %x n'a pas ete adaptee.\n",AdresseVisee);
		ModifieApel(Instruction,IndexAdresse,TableRomEMU+183);
	}

}

void ModifieApel(char *Instruction, int IndexAdresse, unsigned short AdresseDeRemplacement)
{
	unsigned short AdresseVisee;
	AdresseTrouvee = true;
	AdresseVisee = (unsigned char)Instruction[IndexAdresse]+Instruction[IndexAdresse+1]*0x100;
	Instruction[IndexAdresse+1] = (unsigned short)(AdresseDeRemplacement)/0x100;
	Instruction[IndexAdresse  ] = (unsigned short)(AdresseDeRemplacement)-(unsigned char)(Instruction[IndexAdresse+1])*0x100;
}
				

/////////////////////////////////////////////ETAPE 3///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned long checksum = 0;

void fputc_and_sum(int c, FILE* fichier_82p)
{
	fputc(c, fichier_82p);
	checksum += c;
}

void Etape3()
{
	int i;
	fsetpos(fichier_83p,&pos);
	fgetpos(fichier_82p,&pos);
	for (i = 0; i<0x37; i++) fputc(fgetc(fichier_83p),fichier_82p);

	for (i = 0; i<0x48-0x37; i++) fputc_and_sum(fgetc(fichier_83p),fichier_82p);
	//On cherche la premire instruction du programme
	for( i = 0; i<AdresseRelativePremInstr; i++)
		fputc_and_sum(fgetc(fichier_83p),fichier_82p);
	//On charge le programme
	for (i =0; i<TailleProgrammeASM-AdresseRelativePremInstr ; i++)
		fputc_and_sum((unsigned char)Programme[i],fichier_82p);

	//Ecrit le checksum
	fputc((unsigned char)checksum, fichier_82p);
	fputc(checksum >> 8, fichier_82p);

}